@@ -138,6 +138,8 @@ G_DEFINE_ABSTRACT_TYPE (GstObject, gst_object, G_TYPE_INITIALLY_UNOWNED);
 static void
 gst_object_constructed (GObject * object)
 {
+  GST_OBJECT_FLAG_SET (object, GST_OBJECT_FLAG_CONSTRUCTED);
+
   GST_TRACER_OBJECT_CREATED (GST_OBJECT_CAST (object));
 
   ((GObjectClass *) gst_object_parent_class)->constructed (object);
@@ -324,7 +326,7 @@ gst_clear_object (GstObject ** object_ptr)
  * gst_object_replace:
  * @oldobj: (inout) (transfer full) (nullable): pointer to a place of
  *     a #GstObject to replace
- * @newobj: (transfer none) (allow-none): a new #GstObject
+ * @newobj: (transfer none) (nullable): a new #GstObject
  *
  * Atomically modifies a pointer to point to a new object.
  * The reference count of @oldobj is decreased and the reference count of
@@ -349,7 +351,7 @@ gst_object_replace (GstObject ** oldobj, GstObject * newobj)
       newobj ? G_OBJECT (newobj)->ref_count : 0);
 #endif
 
-  oldptr = g_atomic_pointer_get ((gpointer *) oldobj);
+  oldptr = (GstObject *) g_atomic_pointer_get ((gpointer *) oldobj);
 
   if (G_UNLIKELY (oldptr == newobj))
     return FALSE;
@@ -358,7 +360,7 @@ gst_object_replace (GstObject ** oldobj, GstObject * newobj)
     gst_object_ref (newobj);
 
   while (G_UNLIKELY (!g_atomic_pointer_compare_and_exchange ((gpointer *)
-              oldobj, oldptr, newobj))) {
+              oldobj, (gpointer) oldptr, newobj))) {
     oldptr = g_atomic_pointer_get ((gpointer *) oldobj);
     if (G_UNLIKELY (oldptr == newobj))
       break;
@@ -630,7 +632,7 @@ had_parent:
 /**
  * gst_object_set_name:
  * @object: a #GstObject
- * @name: (allow-none): new name of object
+ * @name: (nullable): new name of object
  *
  * Sets the name of @object, or gives @object a guaranteed unique
  * name (if @name is %NULL).
@@ -765,6 +767,36 @@ gst_object_get_parent (GstObject * object)
   return result;
 }
 
+/**
+ * gst_object_get_toplevel:
+ * @object: a #GstObject
+ *
+ * Returns the toplevel parent of @object. This function increases the refcount
+ * of the toplevel object so you should gst_object_unref() it after usage.
+ *
+ * Returns: (transfer full): toplevel of @object, or @object itself if it has no
+ *   parent. unref after usage.
+ *
+ * MT safe. Grabs and releases @object's LOCK.
+ *
+ * Since: 1.28
+ */
+GstObject *
+gst_object_get_toplevel (GstObject * object)
+{
+  GstObject *result = NULL, *parent;
+
+  g_return_val_if_fail (GST_IS_OBJECT (object), NULL);
+
+  result = gst_object_ref (object);
+  while ((parent = gst_object_get_parent (result))) {
+    gst_object_unref (result);
+    result = parent;
+  }
+
+  return result;
+}
+
 /**
  * gst_object_unparent:
  * @object: a #GstObject to unparent
@@ -1342,7 +1374,7 @@ gst_object_remove_control_binding (GstObject * object,
  *
  * Gets the value for the given controlled property at the requested time.
  *
- * Returns: (nullable): the GValue of the property at the given time,
+ * Returns: (transfer full) (nullable): the GValue of the property at the given time,
  * or %NULL if the property isn't controlled.
  */
 GValue *
@@ -1372,7 +1404,7 @@ gst_object_get_value (GstObject * object, const gchar * property_name,
  * @timestamp: the time that should be processed
  * @interval: the time spacing between subsequent values
  * @n_values: the number of values
- * @values: array to put control-values in
+ * @values: (array length=n_values): array to put control-values in
  *
  * Gets a number of values for the given controlled property starting at the
  * requested time. The array @values need to hold enough space for @n_values of
@@ -1497,3 +1529,23 @@ gst_object_set_control_rate (GstObject * object, GstClockTime control_rate)
 
   object->control_rate = control_rate;
 }
+
+/**
+ * gst_object_call_async:
+ * @object: (transfer none): a #GstObject
+ * @func: (scope async): function to call asynchronously from another thread
+ * @user_data: data to pass to @func
+ *
+ * Equivalent to gst_element_call_async() but this API allows @func to be called
+ * with #GstObject. See also gst_element_call_async()
+ *
+ * Since: 1.28
+ */
+void
+gst_object_call_async (GstObject * object, GstObjectCallAsyncFunc func,
+    gpointer user_data)
+{
+  g_return_if_fail (GST_IS_OBJECT (object));
+
+  _priv_gst_object_call_async (object, (GFunc) func, user_data, NULL);
+}
