@@ -107,10 +107,10 @@ GType gst_aggregator_pad_get_type           (void);
  ***************************/
 
 GST_BASE_API
-GstBuffer * gst_aggregator_pad_pop_buffer   (GstAggregatorPad *  pad);
+GstBuffer * gst_aggregator_pad_pop_buffer   (GstAggregatorPad *  pad) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_BASE_API
-GstBuffer * gst_aggregator_pad_peek_buffer  (GstAggregatorPad *  pad);
+GstBuffer * gst_aggregator_pad_peek_buffer  (GstAggregatorPad *  pad) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_BASE_API
 gboolean    gst_aggregator_pad_drop_buffer  (GstAggregatorPad *  pad);
@@ -121,6 +121,9 @@ gboolean    gst_aggregator_pad_has_buffer   (GstAggregatorPad * pad);
 GST_BASE_API
 gboolean    gst_aggregator_pad_is_eos       (GstAggregatorPad *  pad);
 
+GST_BASE_API
+gboolean    gst_aggregator_pad_is_inactive  (GstAggregatorPad * pad);
+
 /*********************
  * GstAggregator API *
  ********************/
@@ -261,14 +264,42 @@ struct _GstAggregatorClass {
 
   GstFlowReturn     (*flush)          (GstAggregator    *  aggregator);
 
+  /**
+   * GstAggregatorClass::clip:
+   * @self: the #GstAggregator
+   * @aggregator_pad: a #GstAggregatorPad
+   * @buf: (transfer full): a #GstBuffer
+   *
+   * Called when a buffer is received on a sink pad, the task of
+   * clipping it and translating it to the current segment falls
+   * on the subclass. The function should use the segment of data
+   * and the negotiated media type on the pad to perform
+   * clipping of input buffer. This function takes ownership of
+   * buf and should output a buffer or return NULL in
+   * if the buffer should be dropped.
+   *
+   * Returns: (transfer full) (nullable): a #GstBuffer.
+   */
   GstBuffer *       (*clip)           (GstAggregator    *  aggregator,
                                        GstAggregatorPad *  aggregator_pad,
                                        GstBuffer        *  buf);
 
+  /**
+   * GstAggregatorClass::finish_buffer:
+   * @event: (transfer full): a #GstBuffer
+   */
   GstFlowReturn     (*finish_buffer)  (GstAggregator    * aggregator,
                                        GstBuffer        * buffer);
 
   /* sinkpads virtual methods */
+  /**
+   * GstAggregatorClass::sink_event:
+   * @aggregator_pad: a #GstAggregatorPad
+   * @event: (transfer full): a #GstEvent
+   *
+   * Called when an event is received on a sink pad, the subclass
+   * should always chain up.
+   */
   gboolean          (*sink_event)     (GstAggregator    *  aggregator,
                                        GstAggregatorPad *  aggregator_pad,
                                        GstEvent         *  event);
@@ -278,6 +309,13 @@ struct _GstAggregatorClass {
                                        GstQuery         *  query);
 
   /* srcpad virtual methods */
+  /**
+   * GstAggregatorClass::src_event:
+   * @event: (transfer full): a #GstEvent
+   *
+   * Called when an event is received on the src pad, the subclass
+   * should always chain up.
+   */
   gboolean          (*src_event)      (GstAggregator    *  aggregator,
                                        GstEvent         *  event);
 
@@ -297,6 +335,19 @@ struct _GstAggregatorClass {
 
   GstClockTime      (*get_next_time)  (GstAggregator    *  aggregator);
 
+  /**
+   * GstAggregatorClass::create_new_pad:
+   * @self: the #GstAggregator
+   * @templ: the pad template to use
+   * @req_name: (nullable) requested pad name
+   * @caps: (nullable) caps for the pad
+   *
+   * Called when a new pad needs to be created. Allows subclass that
+   * don't have a single sink pad template to provide a pad based
+   * on the provided information.
+   *
+   * Returns: (transfer floating): a new #GstAggregatorPad.
+   */
   GstAggregatorPad * (*create_new_pad) (GstAggregator  * self,
                                         GstPadTemplate * templ,
                                         const gchar    * req_name,
@@ -304,11 +355,21 @@ struct _GstAggregatorClass {
 
   /**
    * GstAggregatorClass::update_src_caps:
-   * @ret: (out) (allow-none):
+   * @ret: (out) (nullable):
    */
   GstFlowReturn     (*update_src_caps) (GstAggregator *  self,
                                         GstCaps       *  caps,
                                         GstCaps       ** ret);
+  /**
+   * GstAggregatorClass::fixate_src_caps:
+   * @caps: (transfer full): a #GstCaps to fixate
+   *
+   * Fixate and return the src pad caps provided. The function takes
+   * ownership of @caps and returns a fixated version of
+   * @caps. @caps is not guaranteed to be writable.
+   *
+   * Returns: (transfer full): the fixated caps #GstCaps.
+   */
   GstCaps *         (*fixate_src_caps) (GstAggregator *  self,
                                         GstCaps       *  caps);
   gboolean          (*negotiated_src_caps) (GstAggregator *  self,
@@ -322,6 +383,14 @@ struct _GstAggregatorClass {
 
   gboolean          (*negotiate) (GstAggregator * self);
 
+  /**
+   * GstAggregatorClass::sink_event_pre_queue:
+   * @aggregator_pad: a #GstAggregatorPad
+   * @event: (transfer full): a #GstEvent
+   *
+   * Called when an event is received on a sink pad before queueing up
+   * serialized events. The subclass should always chain up (Since: 1.18).
+   */
   GstFlowReturn     (*sink_event_pre_queue)     (GstAggregator    *  aggregator,
                                                  GstAggregatorPad *  aggregator_pad,
                                                  GstEvent         *  event);
@@ -330,8 +399,32 @@ struct _GstAggregatorClass {
                                                  GstAggregatorPad *  aggregator_pad,
                                                  GstQuery         *  query);
 
+  /**
+   * GstAggregatorClass::finish_buffer_list:
+   * @bufferlist: (transfer full): a #GstBufferList
+   *
+   * Optional. Equivalent of #GstAggregatorClass::finish_buffer for
+   * buffer lists.
+   *
+   * Since: 1.18
+   */
+  GstFlowReturn     (*finish_buffer_list) (GstAggregator    * aggregator,
+                                           GstBufferList    * bufferlist);
+  /**
+   * GstAggregatorClass::peek_next_sample:
+   * @aggregator_pad: a #GstAggregatorPad
+   *
+   * Returns: (transfer full) (nullable): the #GstSample.
+   *
+   * See gst_aggregator_peek_next_sample().
+   *
+   * Since: 1.18
+   */
+  GstSample *       (*peek_next_sample)         (GstAggregator *aggregator,
+                                                 GstAggregatorPad * aggregator_pad);
+
   /*< private >*/
-  gpointer          _gst_reserved[GST_PADDING_LARGE-3];
+  gpointer          _gst_reserved[GST_PADDING_LARGE-5];
 };
 
 /************************************
@@ -356,6 +449,14 @@ GST_BASE_API
 GstFlowReturn  gst_aggregator_finish_buffer         (GstAggregator                *  aggregator,
                                                      GstBuffer                    *  buffer);
 
+GST_BASE_API
+GstFlowReturn  gst_aggregator_finish_buffer_list    (GstAggregator                *  aggregator,
+                                                     GstBufferList                *  bufferlist);
+
+GST_BASE_API
+gboolean       gst_aggregator_push_src_event        (GstAggregator                *  aggregator,
+                                                     GstEvent                     *  event);
+
 GST_BASE_API
 void           gst_aggregator_set_src_caps          (GstAggregator                *  self,
                                                      GstCaps                      *  caps);
@@ -375,7 +476,7 @@ GST_BASE_API
 GstClockTime  gst_aggregator_get_latency           (GstAggregator                 *  self);
 
 GST_BASE_API
-GstBufferPool * gst_aggregator_get_buffer_pool     (GstAggregator                 * self);
+GstBufferPool * gst_aggregator_get_buffer_pool     (GstAggregator                 * self) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_BASE_API
 void            gst_aggregator_get_allocator       (GstAggregator                 * self,
@@ -389,6 +490,31 @@ GST_BASE_API
 void            gst_aggregator_update_segment       (GstAggregator                * self,
                                                      const GstSegment             * segment);
 
+GST_BASE_API
+GstSample     * gst_aggregator_peek_next_sample     (GstAggregator *self,
+                                                     GstAggregatorPad * pad) G_GNUC_WARN_UNUSED_RESULT;
+
+GST_BASE_API
+void            gst_aggregator_selected_samples     (GstAggregator                * self,
+                                                     GstClockTime                   pts,
+                                                     GstClockTime                   dts,
+                                                     GstClockTime                   duration,
+                                                     GstStructure                 * info);
+
+GST_BASE_API
+void            gst_aggregator_set_ignore_inactive_pads (GstAggregator * self,
+                                                         gboolean ignore);
+
+GST_BASE_API
+gboolean        gst_aggregator_get_ignore_inactive_pads (GstAggregator * self);
+
+GST_BASE_API
+gboolean        gst_aggregator_get_force_live       (GstAggregator *self);
+
+GST_BASE_API
+void            gst_aggregator_set_force_live       (GstAggregator *self,
+                                                     gboolean force_live);
+
 /**
  * GstAggregatorStartTimeSelection:
  * @GST_AGGREGATOR_START_TIME_SELECTION_ZERO: Start at running time 0.
@@ -396,6 +522,8 @@ void            gst_aggregator_update_segment       (GstAggregator
  * the first buffer that is received.
  * @GST_AGGREGATOR_START_TIME_SELECTION_SET: Start at the running time
  * selected by the `start-time` property.
+ * @GST_AGGREGATOR_START_TIME_SELECTION_NOW: Start at the current running time
+ * when reaching %GST_STATE_PLAYING.
  *
  * Since: 1.18
  */
@@ -403,7 +531,16 @@ typedef enum
 {
   GST_AGGREGATOR_START_TIME_SELECTION_ZERO,
   GST_AGGREGATOR_START_TIME_SELECTION_FIRST,
-  GST_AGGREGATOR_START_TIME_SELECTION_SET
+  GST_AGGREGATOR_START_TIME_SELECTION_SET,
+
+  /**
+   * GST_AGGREGATOR_START_TIME_SELECTION_NOW:
+   *
+   * Start at the current running time when reaching %GST_STATE_PLAYING.
+   *
+   * Since: 1.28
+   */
+  GST_AGGREGATOR_START_TIME_SELECTION_NOW,
 } GstAggregatorStartTimeSelection;
 
 GST_BASE_API
