@@ -29,7 +29,19 @@
 
 /**
  * SECTION:gstbitreader
+ * @title: GstBitReader
  * @short_description: Reads any number of bits from a memory buffer
+ * @symbols:
+ * - gst_bit_reader_skip_unchecked
+ * - gst_bit_reader_skip_to_byte_unchecked
+ * - gst_bit_reader_get_bits_uint8_unchecked
+ * - gst_bit_reader_peek_bits_uint8_unchecked
+ * - gst_bit_reader_get_bits_uint16_unchecked
+ * - gst_bit_reader_peek_bits_uint16_unchecked
+ * - gst_bit_reader_get_bits_uint32_unchecked
+ * - gst_bit_reader_peek_bits_uint32_unchecked
+ * - gst_bit_reader_get_bits_uint64_unchecked
+ * - gst_bit_reader_peek_bits_uint64_unchecked
  *
  * #GstBitReader provides a bit reader that can read any number of bits
  * from a memory buffer. It provides functions for reading any number of bits
@@ -51,7 +63,7 @@
 GstBitReader *
 gst_bit_reader_new (const guint8 * data, guint size)
 {
-  GstBitReader *ret = g_slice_new0 (GstBitReader);
+  GstBitReader *ret = g_new0 (GstBitReader, 1);
 
   ret->data = data;
   ret->size = size;
@@ -71,7 +83,7 @@ gst_bit_reader_free (GstBitReader * reader)
 {
   g_return_if_fail (reader != NULL);
 
-  g_slice_free (GstBitReader, reader);
+  g_free (reader);
 }
 
 /**
